from PIL import Image
import numpy as np
import pandas as pd


class BaseStorage:
    def __init__(self):
        pass

    def create(self, data, name, **kwargs):
        if isinstance(data, str):  # puedo asumir que siempre será un path?
            return self.create_from_path(data, name)
        elif isinstance(data, Image.Image):
            return self.create_from_image(data, name)
        elif isinstance(data, np.ndarray):
            ext = name.split(".")[-1]
            if ext in ["tif", "tiff"]:
                if "ds" in kwargs:
                    return self.create_from_rasterio(data, name, kwargs["ds"])
                raise TypeError("Missing ds argument")
            elif ext == "npy":
                return self.create_from_array(data, name)
            else:
                raise TypeError("Not a valid array type")
        elif isinstance(data, pd.core.frame.DataFrame):
            ext = name.split(".")[-1]
            if ext == "csv":
                return self.create_from_csv(data, name)
            elif ext == "json":
                return self.create_from_json(data, name)
            else:
                raise TypeError("Not a valid dataframe type")
        else:
            raise TypeError("Not a valid type")

    def create_from_data(self, data, path):
        with open(path, "wb") as f:
            f.write(data.read())
        return path

    def read(self, name):
        ext = name.split(".")[-1]
        if ext == "npy":
            return self.read_from_array(name)
        elif ext in ["tif", "tiff"]:
            return self.read_from_rasterio(name)
        elif ext == "csv":
            return self.read_from_csv(name)
        elif ext == "json":
            return self.read_from_json(name)
        raise TypeError("Not a valid type")

    def update(self):
        pass

    def delete(self):
        pass

    def list(self, pattern="*"):
        pass



from .sentinelhub import SHExplorer
import geopandas as gpd


def explore_satellite_images(time_interval, sensor="S2L2A", **kargs):
    # aoi = retrieve_aoi_from_location(location)
    gdf = gpd.GeoDataFrame.from_features(
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {},
                    "geometry": {
                        "coordinates": [
                            [
                                [2.058000718868186, 41.46183613708533],
                                [2.058000718868186, 41.4318016264732],
                                [2.097234913854237, 41.4318016264732],
                                [2.097234913854237, 41.46183613708533],
                                [2.058000718868186, 41.46183613708533],
                            ]
                        ],
                        "type": "Polygon",
                    },
                }
            ],
        },
        crs=4326,  # validate coords are lat/lon !
    )
    explorer = SHExplorer(time_interval, sensor, **kargs)
    results = explorer.search(gdf)
    return results



import os
import requests


def deploy_and_run_cloud(dir, config, typer):
    # TODO: check if project exists and confirm to overwrite
    # send to api
    typer.echo(f"Deploying...")
    if "agents" in config:
        typer.echo(f"Deploying agents...")
        for agent in config["agents"]:
            typer.echo(f"Deploying agent '{agent['name']}'...")
            agent_dir = dir / "agents" / agent["name"]
            has_requirements = "requirements.txt" in os.listdir(agent_dir)
            has_env = ".env" in os.listdir(agent_dir)
            response = requests.post(
                "http://localhost:8000/deploy/agent",
                files={
                    "agent": open(agent_dir / "main.py", "rb"),
                    "requirements": open(agent_dir / "requirements.txt", "rb")
                    if has_requirements
                    else None,
                    "env": open(agent_dir / ".env", "rb") if has_env else None,
                },
                data={
                    "name": agent["name"],
                },
            )
            if response.status_code == 200:
                typer.echo("Agent deployed successfully!")
            else:
                typer.echo("Something went wrong.")
                typer.echo(response.json()["detail"])



from .sentinelhub import SHS2L2ADownloader, SHS2L1CDownloader, SHS1Downloader
import geopandas as gpd
import shutil


# def download_satellite_image(location, date, sensor, options):
def download_satellite_image(date, sensor, storage):
    # aoi = retrieve_aoi_from_location(location)
    gdf = gpd.GeoDataFrame.from_features(
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {},
                    "geometry": {
                        "coordinates": [
                            [
                                [2.058000718868186, 41.46183613708533],
                                [2.058000718868186, 41.4318016264732],
                                [2.097234913854237, 41.4318016264732],
                                [2.097234913854237, 41.46183613708533],
                                [2.058000718868186, 41.46183613708533],
                            ]
                        ],
                        "type": "Polygon",
                    },
                }
            ],
        },
        crs=4326,  # validate coords are lat/lon !
    )
    download_dir = "/tmp/sentinelhub"
    if sensor == "S2L2A":
        downloader = SHS2L2ADownloader(download_dir)
    elif sensor == "S2L1C":
        downloader = SHS2L1CDownloader(download_dir)
    elif sensor == "S1":
        downloader = SHS1Downloader(download_dir)
    else:
        raise Exception(f"sensor {sensor} not supported")
    dst_path = downloader.download(gdf, date)
    dst_path = storage.create(dst_path, name=f"{sensor}_{date}.tif")
    shutil.rmtree(download_dir)
    return dst_path
